/**
 * Gulp Require Modules
 */
var gulp             = require( 'gulp' ),
    browserSync      = require( 'browser-sync' ),
    uglify           = require( 'gulp-uglify' ),
    jshint           = require( 'gulp-jshint' ),
    concat           = require( 'gulp-concat' ),
    sass             = require( 'gulp-ruby-sass' ),
    stripCssComments = require( 'gulp-strip-css-comments' ),
    autoprefixer     = require( 'gulp-autoprefixer' ),
    cache            = require( 'gulp-cached' ),
    remember         = require( 'gulp-remember' ),
    rename           = require( 'gulp-rename' ),
    imageop          = require( 'gulp-image-optimization' );
    clean            = require( 'gulp-clean' ),
    zip              = require( 'gulp-zip' );

/**
 * Paths
 */

var paths = {

  scripts: [
    '!assets/js/src/vendor/**/*.js',
    'bower_components/slick-carousel/slick/slick.js',
    'bower_components/matchHeight/jquery.matchHeight-min.js',
    'bower_components/parallax.js/parallax.min.js',
    'assets/js/src/main.js',
    'assets/js/src/**/*.js',
  ],

  styles: [
    'assets/scss/style.scss',
    'assets/scss/*.scss',
    'assets/scss/!(vendor|bourbon|themes)/**/*.scss',
  ],

  vendorStyles: ['assets/scss/vendor/**/*.scss'],

  images: ['assets/images/**/*.png', 'assets/images/**/*.jpg', 'assets/images/**/*.jpeg', 'assets/images/**/*.svg', 'assets/images/**/*.gif', ],

  zipSrc: [
    '!{.gitignore,sftp-config.json}',
    '!node_modules', '!node_modules/**',
    '!dist', '!dist/**',
    '!bower_components', '!bower_components/**',
    './**',
  ],

  zipDist: 'dist',

}

/**
 * Theme Scripts
 * 
 * Concatenates & minifies various scripts of
 * the theme.
 */
gulp.task( 'scripts', function() {

  function jsError( err ) { 
    console.error( 'Error', err.message, '| at line #', err.lineNumber );
  }

  return gulp.src( paths.scripts )
    .on('error', function( err ) {
      jsError( err );
    })
    .pipe( cache('scripts') )
    .pipe( remember('scripts') )
    .pipe( concat( 'main.js' ) )
    .pipe( gulp.dest( 'assets/js/dist/' ) )
    .pipe( rename('main.min.js') )
    .on('error', function( err ) {
      jsError( err );
    })
    .pipe( uglify() )
    .pipe( gulp.dest('assets/js/dist/') )
    .pipe( browserSync.stream( { match: '**/*.js' } ) );

});

/**
 * Task: Theme Sass
 *
 * Runs on theme.scss file only. Exclude any
 * other scss file expect theme.scss partials
 * from the task.
 */
gulp.task( 'style-sass', function() {

  return sass( paths.styles, { style: 'expanded' })
    .on( 'error', sass.logError )
    .pipe( cache( 'sassCache' ) )
    .pipe( remember( 'sassCache' ) )
    .pipe( autoprefixer({
      browsers: [ 'last 2 versions', 'ie 8', 'ie 9' ]
    }))
    .pipe( gulp.dest( './' ) )
    .pipe( browserSync.stream( { match: '**/*.css' } ) );

});

/**
 * Task: Vendor Sass
 *
 * Runs on SCSS vandors only.
 */
gulp.task( 'vendor-sass', function() {

  return sass( paths.vendorStyles, { style : 'compressed' } )
    .on('error', sass.logError)
    .pipe( cache('sassCache') )
    .pipe( autoprefixer({
      browsers: ['last 2 versions', 'ie 8', 'ie 9']
    }))
    .pipe( gulp.dest('assets/css/vendor/') )
    .pipe( browserSync.stream( { match: '**/*.css' } ) );

});

/**
 * Task: Sass
 *
 * Runs both tasks 'style-sass' & 'vendor-sass'.
 */
gulp.task( 'sass', [ 'style-sass', 'vendor-sass' ]);

/**
 * Task: Images
 *
 * Optimizes images located under assets
 */
gulp.task( 'images', function( cb ) {

  gulp.src( paths.images ).pipe( imageop( {
    optimizationLevel : 5,
    progressive       : true,
    interlaced        : true,
  })).pipe( gulp.dest( 'assets/images/' ).on( 'end', cb ).on( 'error', cb ) );

});

/**
 * Browser Sync
 */
gulp.task( 'browser-sync', function () {
  
});

/**
 * Zip Files
 * 
 * This task creates a zip file contains necessary WordPress theme
 * files
 */
gulp.task( 'zip', function() {

  return gulp.src( [
    '!{.gitignore,sftp-config.json}',
    '!node_modules', '!node_modules/**',
    '!dist', '!dist/**',
    '!bower_components', '!bower_components/**',
    '!library/shortcodes', '!library/shortcodes/**',
    './**',
  ] )
    .pipe( zip('poetic.zip') )
    .pipe( gulp.dest( 'dist' ) );

});

/**
 * Task: Watch
 * 
 * Watches for 'sass', 'scripts' changes.
 */
gulp.task( 'watch', function() {

  browserSync.init({
    // proxy: 'http://dev.webic.co.uk/itpoetry',
    proxy: 'http://demo.themetomb.com/poetic',
    reloadDelay: 3200
  });

  gulp.watch( paths.vendorStyles, ['vendor-sass'] );
  gulp.watch( paths.styles, ['style-sass'] );
  gulp.watch( 'assets/js/src/**/*.js', ['scripts'] );
  gulp.watch( [ '**/*.php', '**/*.html' ] ).on( 'change', browserSync.reload );

});

/**
 * Task: Clean Unnecessary Files
 *
 * This task cleans unnecessary files like .DS_Store 
 * from the project folder.
 */

gulp.task( 'clean-junk', function () {
  return gulp.src( '**/.DS_Store', { read: false } )
    .pipe( clean() );
});

/**
 * Task: Default
 * 
 * Runs Gulp default task that includes all tasks.
 */
gulp.task( 'default', [ 'clean-junk', 'scripts', 'sass', 'images', 'watch' ] );